##====================================================================##
##==Purpose: To generate 20 sets of training and test data for      ==## 
##==         cross-validation                                       ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *data/CEL/info.250.csv                                        ==##
##==  It is a table containing the followings columns:              ==##
##==  -patient.id: A vector of patients ID                          ==##
##==  -microarray.id: A vector of microarray ID                     ==##
##==  -batch: A vector of batch ID                                  ==##
##==  -hours.si: A vector of the timing which the microarray was    ==##
##==             collected (in hours).                              ==##
##==  *data/Rdata/dat.cfr.Rdata                                     ==##
##==  It contains a R object dat.cfr (data frame) with with columns ==##
##==  corresponding to patient_id, cluster (i.e. ocMOF), batch and  ==##
##==  sex.                                                          ==##
##==                                                                ==##
##==  Additional parameters                                         ==##
##==  -min_hours, e.g. 12                                           ==##
##==  -max_hours, e.g. 250                                          ==##                               
##==                                                                ==##
##==Output:                                                         ==## 
##==  Exports the *.txt which contains the list of microarrays ids  ==##
##==  for the training (subset 1) and test (subest2) data           ==##
##==  data/CEL/splitCVID_subsetSUBSETID_phaseBATCHID                ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##

##Set the seed
set.seed(1234)

#obtain array.id, patient.id, and hours since injury from the data
#min_hours = 12
#max_hours = 250
info.250<-read.table(file="data/CEL/info.250.csv",sep=",")
valid12.250<-which(info.250[,"hours.si"]>=12 & info.250[,"hours.si"]<=250)
microarray.id<-info.250[valid12.250,"microarray.id"]
patient.id<-info.250[valid12.250,"patient.id"]
hours.si<-info.250[valid12.250,"hours.si"]

#three patients to be removed: one with head injury and other two with problematic correlations
rm.id <- c("33426107","25993337","8028773")

#filter patients with less than three time points
unique.id <- unique(patient.id)
for(i in 1:length(unique.id)) if(length(hours.si[patient.id==unique.id[i]])<3) rm.id = c(rm.id,unique.id[i])
select = !(patient.id %in% rm.id)
microarray.id <- microarray.id[select]
patient.id <- patient.id[select]
hours.si <- hours.si[select]

#obtain phase and ocMOF for each patient
load(file="data/Rdata/dat.cfr.Rdata")
phase = dat.cfr$batch[match(patient.id,dat.cfr$patient_id)]
unique.id <- unique(patient.id)
ocMOF = dat.cfr$cluster[match(unique.id,dat.cfr$patient_id)]

#generate 20 random splits; for each split create 8 separate .txt files
for(jj in 1:20){

  #divide all the patients into two subsets such that the ocMOF distribution are similar
  for(ii in 5:1){
    tmp = unique.id[ocMOF==ii]
    np = length(tmp)
    nphalf = ifelse(sample(c(0,1),1)==0,floor(np/2),ceiling(np/2))
    rp = sample(1:np)
    if(ii==5) pat.1= tmp[rp[1:nphalf]] else pat.1=c(pat.1,tmp[rp[1:nphalf]])
    if(ii==5) pat.2= tmp[rp[(nphalf+1):np]] else pat.2=c(pat.2,tmp[rp[(nphalf+1):np]])
  }

  #write array ids in .txt files
  for(ii in 1:4){ 
	write.table(paste("t",microarray.id[phase==ii & patient.id%in%pat.1],sep=""),
				file=paste("data/CEL/split",jj,"_subset1_phase",ii,".txt",sep=""),
				quote=FALSE,row.names=FALSE,col.names=FALSE) 
	write.table(paste("t",microarray.id[phase==ii & patient.id%in%pat.2],sep=""),
				file=paste("data/CEL/split",jj,"_subset2_phase",ii,".txt",sep=""),
				quote=FALSE,row.names=FALSE,col.names=FALSE) 
  }
  print(sprintf("split %d done...",jj))

}
