##====================================================================##
##==Purpose: Functions for Spearman analysis, including adjusted    ==##
##==         Spearman analysis.                                     ==## 
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu             ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##


spear.pvalue <- function(X,Y,n) {
  # This is a function written by
  # Keyur Desai to perform Spearman
  # rank correlation tests rapidly.
  
  R = spear.stat(X,Y)
  dfr = n - 2
  R2 <- R^2
  Fstat <- R2 * dfr / (1 - R2)
  P <- 1 - pf(Fstat, 1, dfr)
  return(P)
}

spear.stat <- function(X,Y) {
  # This is a function written by
  # Keyur Desai to perform Spearman
  # rank correlation tests rapidly.
  
  X = apply(X,1,rank)
  Y = rank(Y)
  R = cor(X,Y)

  return(R)
}

spear.cor.pvalue <- function(X,Y,n) {
# This is a function written by
# Keyur Desai to perform Spearman
# rank correlation tests rapidly.
	
	R = spear.stat(X,Y)
	dfr = n - 2
	R2 <- R^2
	Fstat <- R2 * dfr / (1 - R2)
	P <- 1 - pf(Fstat, 1, dfr)
	return(data.frame(cor=R,pval=P))
}


getp <- function(lr,lr0) {
  ##A function to compute resampled p-values
  ##lr: observed statistics
  ##lr0: resampled statistics
 	
  m <- length(lr)
  v <- c(rep(T,m),rep(F,length(lr0)))
  v <- v[rev(order(c(lr,lr0)))]
  u <- 1:length(v)
  w <- 1:m
  p <- (u[v==TRUE]-w)/(length(lr0))
  p <- p[rank(-lr)]
  return(p)
}


adjSpearman<-function(dat,mod,mod0,res.p=200,modID=pvID,mof,stat=FALSE){
	
  ##A function to do adjusted Spearman analysis
  ##dat: the slope data where columns=patients, rows=genes
  ##mod: the design matrix of the full model
  ##mod0: the design matrix of the null model
  ##res.p: the number of resampling iteration for the p-value computation
  ##modID: a scalar of vector indicating which columns of mod correspond
  ##       to the primary variable (e.g. ocMOF).
  ##mof: the vector corresponding to the primary variable (e.g. ocMOF)
  

  if(res.p>0){        
    ##Regress out ALL VARIABLES from dat      
	par.est <- (dat %*% mod %*% solve(t(mod) %*% mod))
	fit <- par.est %*% t(mod)
	res.dat <- dat - fit 
	  
    ##Use ocmof as the primary variable         
    pvals <- spear.pvalueResample(dat,res.dat,out.used=mof,mod,mod0,res.no=res.p,modID=modID)  
    return(pvals)
	  
  }else{
	  
    ##Regress out ONLY CONFOUNDERS from dat      
	par.est <- (dat %*% mod %*% solve(t(mod) %*% mod))[,-modID]
	fit <- par.est %*% t(mod[,-modID])
	adj.dat <- dat - fit 
	  
    ##The adjusted degrees of freedom
    n<-nrow(mod)-ncol(mod)
    out <- spear.cor.pvalue(adj.dat,mof,n)
	if(stat){
	  return(out)	
	}else{
      return(out[,"pval"])
	}
  }       
}


spear.pvalueResample <- function(dat,res.dat,out.used,mod,mod0,res.no=5,modID,seed=NULL){
  
  ##A function to compute p-value via resampling
  ##dat: the data (i.e. slope matrix or WPEC) columns=patients, rows=genes
  ##res.dat: the residual data (i.e. remove all variables) columns=patients, rows=genes
  ##out.used: the primary variable of interest (e.g. ocMOF)
  ##mod: the design matrix of the full model
  ##mod0: the design matrix of the null model
  ##res.no: the number of resampling iteration for the p-value computation
  ##modID: a scalar of vector indicating which columns of mod correspond
  ##       to the primary variable (e.g. ocMOF).
  ##seed: a scalar for setting the seed

  if(!is.null(seed))
    set.seed(seed)
  
  n <- dim(res.dat)[2]
  m <- dim(res.dat)[1]
  
  I1<-diag(dim(res.dat)[2])
  
  ##Regress out only the confounders
  H <- (mod %*% solve(t(mod) %*% mod))[,-modID]%*% t(mod[,-modID]) 
  IH<-I1-H
  
  ##The observed statistics
  stats<-spear.stat((dat%*%IH),out.used)  	
  ##Store the resampled statistics
  stats0 <- matrix(0,nrow=m,ncol=res.no)
  
  for(i in 1:res.no){
  
    ##Resampling each row of res.dat
    X0<-t(apply(res.dat,1,sample))  
	adj.dat0 <- X0%*%IH                   
    stats0[,i] <- spear.stat(adj.dat0,out.used)
  
  }
  
  p0 <- getp(abs(stats),as.vector(abs(stats0)))
  
  return(p0)
}
