##====================================================================##
##==Purpose: Reproducibility of the dataset via cross-validation    ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *data/CEL/info.250.csv                                        ==##
##==  Table where the columns are patient ids, microarray ids,      ==##
##==  batch ids and hours since injury.                             ==##
##==  *data/Rdata/dat.cfr.Rdata                                     ==##
##==  dat.cfr, a data frame with columns corresponding to           ==## 
##==  patient_id, cluster (i.e. ocMOF),batch and sex.               ==##
##==  *Additional parameters                                        ==##
##==  -constant, e.g. 10                                            ==##
##==  -min_hours, e.g. 12                                           ==##
##==  -max_hours, e.g. 250                                          ==##  
##==  -min_hours.2, e.g. 12  [Only for the test subset]             ==##
##==  -max_hours.2, e.g. 250 [Only for the test subset]             ==##                                
##==  -K, e.g. 100 (the top K probesets)                            ==##
##==  -n.cross, max 20 (the number of cross-validation)             ==##
##==                                                                ==##
##==Sub-routines:                                                   ==##
##==  *Function to generate the slope matrix                        ==##
##==  code/2_normalization/mtx_linfit.R                             ==##
##==  *Spearman related functions:                                  ==##
##==  code/3_analysis/analysisfunctions.R                           ==##
##==  *Script to get the discovery and validation data              ==##
##==  code/4_reprod_20cv/reproducibility_getData.R                  ==##
##==                                                                ==##
##==Figures for paper:                                              ==##
##==  *Reproducibility with 20 cross-validations                    ==## 
##==  plots/forpaper/Figure_4d_20cv.pdf                             ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##

set.seed(12345)
min_hours = 12
max_hours = 250
min_hours.2 = min_hours
max_hours.2 = max_hours
K = 100
##Testing workflow
n.cross = 20
#n.cross = 3
constant = 10

print(sprintf("Min hour = %d, Max hour = %d, number of top genes = %d, number of cross validations = %d, added constant = %d",min_hours,max_hours,K,n.cross,constant))

##Function to compute the slope/offset, mtx.linfit
source("code/2_normalization/mtx_linfit.R")
#Load the required functions getp, 
source("code/3_analysis/analysisfunctions.R")

#extract all the array ids and corresponding patient ids and hours since injury within min_hours to max_hours
info.250<-read.table(file="data/CEL/info.250.csv",sep=",")
info.250<-info.250[which(info.250[,"hours.si"]>=min_hours & info.250[,"hours.si"]<=max_hours),]
patient.id<-info.250[,"patient.id"]	
microarray.id<-paste("t",info.250[,"microarray.id"],sep="")
hours.si<-info.250[,"hours.si"]
rm(info.250)

##The load the confounders
load(file=paste("data/Rdata/dat.cfr.Rdata"))
L = dat.cfr
rm(dat.cfr)

Pall = matrix(0,K,n.cross)
for(split.id in 1:n.cross){

  print(sprintf("crossvalidating split %d...",split.id))
  
  ##Extract the normalized data, compute WPEC (i.e. slope matrix) and extract the confounders for each subset within a split
  source("code/4_reprod_20cv/reproducibility_getData.R")	
  #Adjusted rank-based Spearman correlation test for discoverydisc data
  mod <- model.matrix( ~ as.factor(cluster)+as.factor(batch)+as.factor(sex),
					  data=data.frame(cluster=ocMOF.disc,batch=phase.disc,sex=sex.disc))
  mod0 <- model.matrix( ~ as.factor(batch)+as.factor(sex.disc),
					   data=data.frame(cluster=ocMOF.disc,batch=phase.disc,sex=sex.disc))	
  #The column indices of the primary variables for mod
  pvID<-c(2:5)
  if(length(unique(ocMOF.disc))<5) stop("Train set has < 5 ocMOF group")
  #Computed just the adjusted correlations to get the ranking---no need for p-values
  rnk.disc = order(adjSpearman(X.disc,mod,mod0,res.p=0,modID=pvID,mof=ocMOF.disc))

  #Adjusted rank-based Spearman correlation test for validation data
  mod <- model.matrix( ~ as.factor(cluster)+as.factor(batch)+as.factor(sex),
  					  data=data.frame(cluster=ocMOF.vali,batch=phase.vali,sex=sex.vali))
  mod0 <- model.matrix( ~ as.factor(batch)+as.factor(sex),
					   data=data.frame(cluster=ocMOF.vali,batch=phase.vali,sex=sex.vali))	
  #The column indices of the primary variables for mod
  pvID<-c(2:5)
  if(length(unique(ocMOF.vali))<5) stop("Test set has < 5 ocMOF group")
  #compute p-values using B resamplings
  #Testing workflow
  pval.vali = adjSpearman(X.vali,mod,mod0,res.p=50,modID=pvID,mof=ocMOF.vali)[rnk.disc[1:K]]
#  pval.vali = adjSpearman(X.vali,mod,mod0,res.p=2,modID=pvID,mof=ocMOF.vali)[rnk.disc[1:K]]
  Pall[,split.id] = pval.vali
  print("--------------------")
  print(sprintf("split %d done...",split.id))
  print("--------------------")

}


#qqplot for reproducibility
pdf("plots/forpaper/Figure_4d_20cv.pdf")
par(mar=c(5,5.5,0.5,0.5),las=1,mgp=c(3.6,1,0),bty="n")
xx = seq(0,1,length=K)
split.id=1
qqplot(xx,Pall[,split.id],type="l",xlim=c(0,1),ylim=c(0,1),col=split.id,xlab="Uniform quantiles",ylab="Observed quantiles",main="",cex.lab=2,cex.axis=2)
for(split.id in 2:n.cross){
  par(new=T,bty="n")
  qqplot(xx,Pall[,split.id],type="l",xlim=c(0,1),ylim=c(0,1),col=split.id,xlab="",ylab="",main="",cex.lab=2,cex.axis=2,xaxt = "n",yaxt="n")
}
abline(0,1,lwd=3,lty=2)
dev.off()
