##====================================================================##
##==Purpose: To get clinical characteristics of the ocMOF subgroups.==##
##==         The p-value that indicate the mean MOF is              ==##
##==         insignificant between ocMOF iii and ocMOF v.           ==## 
##==         Produce information for the table in the main paper.   ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *Clinical variables                                           ==##
##==  *data/clinical_data/TRDB_TRAUMA-PATIENT_CLIN_DATA_RPT_20090126==##
##==  _195655.xls                                                   ==##
##==  *data/Rdata/dat.cfr.Rdata                                     ==##
##==  dat.cfr, a data frame with columns corresponding to           ==## 
##==  patient_id, cluster (i.e. ocMOF),batch and sex.               ==##
##==  *data/Rdata/xDat(marshall_nogcs).Rdata                        ==##
##==  xDat, a data frame with rows corresponding to time, columns   ==##
##==  corresponding to patient_id, and the entries the mof score.   ==##
##==  *"data/Rdata/microarray/",type1,".Rdata"                      ==##
##==  gDat, a matrix with columns corresponding to patients and     ==##
##==  rows the probesets. The entries are the related to gene       ==##
##==  expressions.                                                  ==##
##==                                                                ==##
##==Output:                                                         ==##
##==  *log/information_paper.log                                    ==##
##==  Information provided in the paper                             ==##
##==                                                                ==##
##== R libraries: nnet                                              ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##

library(nnet)

##==Read clinical data to validate the groupings
library(gdata)
clinDatNV<-read.xls("data/clinical_data/TRDB_TRAUMA-PATIENT_CLIN_DATA_RPT_20090126_195655.xls")

load(file=paste("data/Rdata/dat.cfr.Rdata"))
ref<-dat.cfr[,"patient_id"]
mat<-clinDatNV[,1]
id<-match(ref,mat)
clinDatNV<-clinDatNV[id,]

if(!all(clinDatNV[,1]==dat.cfr[,1])) stop("Matching is not working")
varnames<-c("AGE","SEX")
dat.info<-data.frame(dat.cfr,clinDatNV[,varnames])

##The ordering of the ocMOF according to paper ocMOF i (best outcome), ..., ocMOF 5 (worst outcome)
if(!all(dat.info[,1]==clinDatNV[,1])) stop("There is something wrong in the matching")
grp<-dat.info[,"cluster"]

mean.info<-NULL
sd.info<-NULL
pval.info<-NULL

##==Age
i<-"AGE"
mean.info<-rbind(mean.info,tapply(dat.info[,i],dat.info[,"cluster"],mean,na.rm=TRUE))
sd.info<-rbind(sd.info,tapply(dat.info[,i],dat.info[,"cluster"],sd,na.rm=TRUE))
tmp<-NULL
for(j in 1:5){
    valid<-which(dat.info[,"cluster"]==j)
    tmp<-c(tmp,range(dat.info[valid,i],na.rm=TRUE))
}
obj<-summary(lm(dat.info[,i]~dat.info[,"cluster"]))$fstatistic
pval.info<-c(pval.info,cor.test(dat.info[,i],dat.info[,"cluster"],method="spearman")$p.value)

##==Perform the adjusted Spearman analysis 
txt.info<-"microarrayID250.168.batch"
min_hours<-12;
max_hours<-250;
type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")

load(file=paste("data/Rdata/microarray/",type1,".Rdata",sep=""))

##==Filter the microarray data
##Remove the individuals due to quality reasons and death status
gDat<-gDat[,!(as.numeric(colnames(gDat))%in%c(8028773,25993337,33426107))]


##==Other clinical variables
dat.tmp<-cbind(dis.days=clinDatNV[,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"],
               icu.days=(1-pmin(clinDatNV[,"ICU_DAYS"]/clinDatNV[,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"],1))*100,
               icu.ventdays=(1-pmin(clinDatNV[,"ICU_VENTDAYS"]/clinDatNV[,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"],1))*100)
for(i in c("dis.days","icu.days","icu.ventdays")){
	mean.info<-rbind(mean.info,tapply(dat.tmp[,i],dat.info[,"cluster"],mean))
	sd.info<-rbind(sd.info,tapply(dat.tmp[,i],dat.info[,"cluster"],sd))
	tmp<-NULL
	for(j in 1:5){
		valid<-which(dat.info[,"cluster"]==j)
		tmp<-c(tmp,range(dat.tmp[valid,i]))
	}
	obj<-summary(lm(dat.tmp[,i]~dat.info[,"cluster"]))$fstatistic
	pval.info<-c(pval.info,cor.test(dat.tmp[,i],dat.info[,"cluster"],method="spearman")$p.value)
}


names(pval.info)<-c("AGE","dis.days","icu.days","icu.ventdays")
rownames(mean.info)<-c("AGE","dis.days","icu.days","icu.ventdays")
rownames(sd.info)<-c("AGE","dis.days","icu.days","icu.ventdays")
#sd.info<-sd.info/sqrt(nrow(clinDatNV))


tmp<-NULL
for(i in 1:5)
tmp<-cbind(tmp,paste(round(mean.info[,i],1)," +/- ",round(sd.info[,i],1),sep=""))
rownames(tmp)<-names(pval.info)
tmp<-cbind(tmp,pval.info)
colnames(tmp)<-c("ocMOFi","ocMOFii","ocMOFiii","ocMOFiv","ocMOFv","p-value")

sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("============================================================")
print("Information for Supplementary Table 1 (continuous variables)")
print(tmp)
print("============================================================")
sink(file=NULL)


##==The MOF score
load(file="data/Rdata/xDat(marshall_nogcs).Rdata")
ref<-dat.info[,"patient_id"]
mat<-colnames(xDat)
id<-match(ref,mat)
xDat<-xDat[,id]
if(!all(names(grp)==colnames(xDat))) stop("There is something wrong in matching the clinical variables")
if(!all(names(grp)==colnames(dat.info))) stop("There is something wrong in matching the clinical variables")


##==The p-value that indicate the mean MOF is insignificant between ocMOF iii and ocMOF v.
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
ocMOF = dat.info[,"cluster"]
meanmof.7day = apply(xDat[1:7,],2,mean)
print("==============================================================================================")
print("Two-sample t-test p-value for the first 7-days mean Marshall score between ocMOF 3 and ocMOF 5")
print(t.test(meanmof.7day[ocMOF==3],meanmof.7day[ocMOF==5])$p.value)
print("==============================================================================================")
sink(file=NULL)


##MOF score >6, same as genomic storm
mof1<-NULL
for(i in 1:ncol(xDat)){
	tmp<-which(xDat[,i]>6)
	if(length(tmp)>0){	
		mof1<-c(mof1,min(as.numeric(rownames(xDat)[tmp])))
	}else{
		mof1<-c(mof1,NA)
		
	}
}
##Whether the person has MOF
no.mof<-apply(xDat,2,function(x) all(x<=6,na.rm=TRUE))
if(!all(no.mof==is.na(mof1)))  stop("The figures do not match up")  
if(!all(names(no.mof)==dat.info[,1]))  stop("The figures do not match up")  

ii<-c("VENT_ASSOCIATED_PNEUMONIA","NON_INFECT_COMPLICATION","SURGICAL_SITE_INFECTION","NOSOCOMIAL_INFECTION","ICU_TRACH","DEATH_OR_DISCHARGE_DAY_SNC_INJ","FINAL_DISP")
print(summary(clinDatNV[,ii]))

##info for the paper to show linear trend in ocMOF
logisticPval<-function(clinDatNV,i,bID,grp){
	dat<-as.data.frame(table(out=as.numeric(ifelse(clinDatNV[,i]==bID,1,0)),ocMOF=as.numeric(grp)))
	dat[,"out"]<-as.numeric(as.character(dat[,"out"]))
	dat[,"ocMOF"]<-as.numeric(as.character(dat[,"ocMOF"]))
	dat<-merge(dat[dat[,"out"]==0,2:3],dat[dat[,"out"]==1,2:3],by="ocMOF")
    ##Fit the model	
	glm1 <- glm(cbind(Freq.x,Freq.y)~ocMOF, family=binomial, data=dat)
#	print(summary(glm1))
	glm0 <- glm(cbind(Freq.x,Freq.y)~1,     family=binomial, data=dat)
	return(anova(glm0,glm1,test="Chisq")$P[2])
}

set.seed(123456)
##This is not the first time sink is used
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("========================================================")
print("Information for Supplementary Table 1 (in percentage, %)")

print("==SEX")
print("count")
i<-"sex"
print(table(dat.info[,"cluster"],dat.info[,i]))
print("proportion")
print(round(prop.table(table(dat.info[,"cluster"],dat.info[,i]),margin=1)*100))
#print(pvalue(independence_test(table(grp=ordered(dat.info[,"cluster"]),y=dat.info[,i]), teststat = "quad",scores = list(grp = 1:5))))
print(logisticPval(dat.info,i=i,bID="M",dat.info[,"cluster"]))


##Generate the data for regressing clinical variable on principal components	
print("==BATCH")
print("count")
i<-"batch"
print(table(dat.info[,i],dat.info[,"cluster"]))
print("proportion")
print(round(prop.table(table(dat.info[,"cluster"],dat.info[,i]),margin=1)*100))

tmp<-data.frame(y=unlist(dat.info[,i]),dat.info[,"cluster"])
##Generate the statistics	
model1<-multinom(y~.,data=tmp,maxit=10000000)
model0<-multinom(y~1,data=tmp,maxit=10000000)

if(model1$convergence!=0) stop("convergence not obtained for non-null model")
if(model0$convergence!=0) stop("convergence not obtained for null model")
df<-length(summary(model1)$coefficients)-length(summary(model0)$coefficients)
p.tmp<-1-pchisq(model0$deviance-model1$deviance,df=df)
print(p.tmp)

print("==VENT_ASSOCIATED_PNEUMONIA")
print("count")
print(table(grp,clinDatNV[,"VENT_ASSOCIATED_PNEUMONIA"]))
print("proportion")
print(round(prop.table(table(grp,clinDatNV[,"VENT_ASSOCIATED_PNEUMONIA"]),margin=1)*100))
#print(pvalue(independence_test(table(grp=ordered(grp),y=clinDatNV[,"VENT_ASSOCIATED_PNEUMONIA"]), teststat = "quad",scores = list(grp = 1:5))))
print(logisticPval(clinDatNV,i="VENT_ASSOCIATED_PNEUMONIA",bID="01: Yes",grp))

print("==NON_INFECT_COMPLICATION")
print("count")
print(table(grp,clinDatNV[,"NON_INFECT_COMPLICATION"]))
print("proportion")
print(round(prop.table(table(grp,clinDatNV[,"NON_INFECT_COMPLICATION"]),margin=1)*100))
#print(pvalue(independence_test(table(grp=ordered(grp),y=clinDatNV[,"NON_INFECT_COMPLICATION"]), teststat = "quad",scores = list(grp = 1:5))))
print(logisticPval(clinDatNV,i="NON_INFECT_COMPLICATION",bID="01: Yes",grp))

print("==SURGICAL_SITE_INFECTION")
print("count")
print(table(grp,clinDatNV[,"SURGICAL_SITE_INFECTION"]))
print("proportion")
print(round(prop.table(table(grp,clinDatNV[,"SURGICAL_SITE_INFECTION"]),margin=1)*100))
#print(pvalue(independence_test(table(grp=ordered(grp),y=clinDatNV[,"SURGICAL_SITE_INFECTION"]), teststat = "quad",scores = list(grp = 1:5))))
print(logisticPval(clinDatNV,i="SURGICAL_SITE_INFECTION",bID="01: Yes",grp))

print("==NOSOCOMIAL_INFECTION")
print("count")
print(table(grp,clinDatNV[,"NOSOCOMIAL_INFECTION"]))
print("proportion")
print(round(prop.table(table(grp,clinDatNV[,"NOSOCOMIAL_INFECTION"]),margin=1)*100))
#print(pvalue(independence_test(table(grp=ordered(grp),y=clinDatNV[,"NOSOCOMIAL_INFECTION"]), teststat = "quad",scores = list(grp = 1:5))))
print(logisticPval(clinDatNV,i="NOSOCOMIAL_INFECTION",bID="01: Yes",grp))

print("==ICU_TRACH")
print("count")
print(table(grp,clinDatNV[,"ICU_TRACH"]))
print("proportion")
print(round(prop.table(table(grp,clinDatNV[,"ICU_TRACH"]),margin=1)*100))
#print(pvalue(independence_test(table(grp=ordered(grp),y=clinDatNV[,"ICU_TRACH"]), teststat = "quad",scores = list(grp = 1:5))))
print(logisticPval(clinDatNV,i="ICU_TRACH",bID="01: Yes",grp))


print("==Death")
print("Death day range")
print(range(clinDatNV[,"DEATH_DAY_SNC_INJ"],na.rm=TRUE))
print("count")
print(table(grp,clinDatNV[,"FINAL_DISP"]=="09: Death"))
print("proportion")
print(round(prop.table(table(grp,clinDatNV[,"FINAL_DISP"]=="09: Death"),margin=1)*100))
#print(pvalue(independence_test(table(grp=ordered(grp),y=clinDatNV[,"FINAL_DISP"]=="09: Death"), teststat = "quad",scores = list(grp = 1:5))))
print(logisticPval(clinDatNV,i="FINAL_DISP",bID="09: Death",grp))

print("==MOF")
print("count")
print(table(grp,no.mof))
print("proportion")
print(round(prop.table(table(grp,no.mof),1)*100))
if(all(names(no.mof)==dat.info[,1])){
    dat.info<-cbind(dat.info,no.mof=no.mof)
}else{
	stop("There is something wrong")
}
#print(pvalue(independence_test(table(grp=ordered(grp),y=no.mof), teststat = "quad",scores = list(grp = 1:5))))
print(logisticPval(dat.info,i="no.mof",bID=TRUE,grp))
print("=====================================")

print("=====================================================")
print("Information about the death days for ocMOF v and iv")
print("==ocMOF v")
print(summary(clinDatNV[grp==5,"DEATH_DAY_SNC_INJ"]))
print("==ocMOF iv")
tmp<-clinDatNV[grp==4,"DEATH_DAY_SNC_INJ"]
print(tmp[!is.na(tmp)])
print("=====================================================")

##info for the paper to show the ranking of ocMOF
print("========================================")
print("Age and gender information for the paper")
print("Range of age")
print(range(clinDatNV[,"AGE"]))
print("SEX")
print(table(clinDatNV[,"SEX"]))
print("========================================")

##ocMOF groupings of the three patients removed
print("=========================================================")
print("Two patients removed due to QA and one due to head injury")
dat.cfr[dat.cfr[,1]%in%c(8028773,25993337,33426107),]
clinDatNV[clinDatNV[,1]%in%c(8028773,25993337,33426107),c("PATIENT_ID","PRIMARY_DEATH_CAUSE")]
print("=========================================================")

##ocMOF distribution of 126 patients
print("==================================")
print("ocMOF distribution of 126 patients")
table(dat.cfr[dat.cfr[,1]%in%colnames(gDat),"cluster"])
print("==================================")

sink(file=NULL)
