##====================================================================##
##==Purpose: To get the probesets of HLA-DR among the MHC-II genes  ==##
##==         To get the expression trajectories associated with the ==##
##==         gene sets                                              ==##
##==         To get the boxplot of slopes, dominant trajectories and==##
##===        counts of up/down for the various gene sets            ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  These are used to identify the probsets in the gene sets      ==##
##==  *type1                                                        ==##
##==  A string indicating the gene expression of interest           ==## 
##==  e.g.                                                          ==##
##==  txt.info<-"microarrayID250.168.batch"                         ==##
##==  min_hours<-12;                                                ==##
##==  max_hours<-250;                                               ==##
##==  type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")", ==##
##==  sep="")                                                       ==##
##==  *"data/Rdata/microarray/",type1,".Rdata"                      ==##
##==  gDat, a matrix with columns corresponding to patients and     ==##
##==  rows the probests. The entries are the related to gene        ==##
##==  expressions (i.e. slope/WPEC).                                ==##
##==  *data/Rdata/dat.cfr.Rdata                                     ==##
##==  dat.cfr, a data frame with columns corresponding to           ==## 
##==  patient_id, cluster (i.e. ocMOF),batch and sex.               ==##
##==  *txt.info                                                     ==##
##==  Specifying name of the *.txt file which contains the          ==##        
##==  microarrays id. For example, the microarrays for batch is in  ==##
##==  "microarrayID250.168.batch1.txt", then txt.info is            ==##
##==  "microarrayID250.168.batch"                                   ==##
##==  These are used to extract the expression trajectories of the  ==## 
##==  gene sets                                                     ==##
##==  *data/Rdata/microarray/txt.info.Rdata                         ==##
##==  -Xraw: A matrix with rows corresponding to probesets, and the ==##
##==         columns the microarrays of patients at different time  ==##
##==         points.                                                ==##
##==  The follow R objects are used to identify the columns of Xraw.==##  
##==  -patient.id: A vector of patients ID                          ==##
##==  -microarray.id: A vector of microarray ID                     ==##
##==  -batch: A vector of batch ID                                  ==##
##==  -hours.si: A vector of the timing which the microarray was    ==##
##==             collected (in hours).                              ==##
##==                                                                ==##
##==Output:                                                         ==##
##==  *log/information_paper.log                                    ==##
##==  Information provided in the paper                             ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##        
##==       kdesai@princeton.edu                                     ==##
##====================================================================##

constant<-10
txt.info<-"microarrayID250.168.batch"
min_hours<-12;
max_hours<-250;
type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")

##Getting WPEC and ocMOF
load(file=paste("data/Rdata/microarray/",type1,".Rdata",sep=""))

##==Filter the microarray data
##Remove the individuals due to quality reasons and death status
gDat<-gDat[,!(as.numeric(colnames(gDat))%in%c(8028773,25993337,33426107))]

##==The load the confounders
load(file=paste("data/Rdata/dat.cfr.Rdata"))

##================================================##
##==Order the patient_id according to microarray==##
##================================================##
ref<-colnames(gDat)
mat<-dat.cfr[,"patient_id"]
id<-match(ref,mat)
cDat<-dat.cfr[id,]
if(!all(cDat[,"patient_id"]==ref)) stop("confounders and gene expression not ordered correctly")

##The probsets associated with HLA-DR
hla.dr<-c("208894_at","209312_x_at","215193_x_at","204670_x_at","221491_x_at")

##Information regarding microarray id, patient id, hours and batch, and the probesets for 126 patients
load(file=paste("data/Rdata/microarray/",txt.info,".Rdata",sep=""))
id.126<-which(patient.id%in%colnames(gDat))
Xraw<-Xraw[,id.126]
patient.id<-patient.id[id.126]
microarray.id<-microarray.id[id.126]
hours.si<-hours.si[id.126]
ocMOF.id<-cDat[match(patient.id,cDat[,"patient_id"]),"cluster"]
rm(list=c("id.126"))

##Get the gene expression associated with HLD-DR
pID<-hla.dr
Xraw.id<-which(rownames(Xraw)%in%pID)
gDatP<-data.frame(id=rep(patient.id,times=length(pID)),
                  grp=rep(ocMOF.id,times=length(pID)),
                  time=rep(hours.si,times=length(pID)),
                  exp=log2(c(t(Xraw[Xraw.id,]))+constant),
                  probeID=rep(rownames(Xraw)[Xraw.id],each=length(patient.id)),
                  stringsAsFactors=FALSE)

##==================================================================================##
##==Computing the difference in gene expression on the second and third time point==##
##==================================================================================##
pIds<-pID

gDatP<-gDatP[order(gDatP[,"id"],gDatP[,"probeID"],gDatP[,"time"]),]
genelist<-unique(gDatP[,5])
  
##Initializing the first gene
g<-genelist[1]
valid<-gDatP[,5]%in%g
  
sdat<-tapply(gDatP[valid,"exp"],as.factor(gDatP[valid,1]),
                              function(x){
                              	tmp<-NULL
                               	if(length(x)>=3){
                                	tmp<-c(tmp,x[3]-x[2])
                               	}
                               	n<-(1-length(tmp))
                               	return(c(tmp,rep(NA,n)))
                               	})
sdat<-data.frame(id=names(sdat),g=sdat,stringsAsFactors=FALSE)
colnames(sdat)[2]<-g
gDatDiff<-sdat

##Compute the other genes
for(g in genelist[-1]){                  
  valid<-gDatP[,5]%in%g
  sdat<-tapply(gDatP[valid,"exp"],as.factor(gDatP[valid,1]),
                                function(x){
                                	tmp<-NULL
                                	if(length(x)>=3){
                                	tmp<-c(tmp,x[3]-x[2])
                                	}
                                	n<-(1-length(tmp))
                                	return(c(tmp,rep(NA,n)))
                                	})
 
  sdat<-data.frame(id=names(sdat),g=sdat,stringsAsFactors=FALSE)
  colnames(sdat)[2]<-g
  	
  gDatDiff<-merge(gDatDiff,sdat,by=1,all=T)                  
}


##=====================================================================##
##==Computing the time difference on the second and third time points==##
##=====================================================================##

sdat1<-do.call("rbind",tapply(gDatP[-1,"time"],as.factor(gDatP[-1,1]),
                              function(x){
                               	x<-sort(unique(x))
                               	x<-x[!is.na(x)]
                               	tmp<-NULL
                               	if(length(x)>=2){
                                	tmp<-c(tmp,x[1],x[2]-x[1])
                               	}
                               	if(length(x)>=3){ 
                                	tmp<-c(tmp,x[2],x[3]-x[2])
                               	} 
                               	if(length(x)>=4){ 
                                	tmp<-c(tmp,x[3],x[4]-x[3])
                               	}
                               	n<-(6-length(tmp))
                               	return(c(tmp,rep(NA,n)))
                               	}))


##THe time and gene expression differ in the number of patients because two patients had only 1 microarray reading
if(all(gDatDiff[,1]!=rownames(sdat1))){
	id<-match(gDatDiff[,1],rownames(sdat1))
	if(!all(rownames(sdat1)[id]==gDatDiff[,1])) stop("Error in matching")
	sdat1<-sdat1[id,]
}
sdat1<-sdat1[(rownames(sdat1)%in%gDatDiff[,1]),]
if(!all(gDatDiff[,1]%in%rownames(sdat1))) stop("Things are not matching up")

##================================================##
##==Order the patient_id according to microarray==##
##================================================##
ref<-colnames(gDat)

mat<-gDatDiff[,1]
id<-match(ref,mat)
gDat1<-gDatDiff[id,]
if(!all(gDat1[,1]==ref)) stop("Error in matching")

mat<-rownames(sdat1)
id<-match(ref,mat)
sdat1<-sdat1[id,]
all(rownames(sdat1)==ref)

##Now get the slopes for gDat1
gDat1[,-1]<-gDat1[,-1]/sdat1[,4]


##Using probesets by pIds
id<-which(colnames(gDat1)%in%pIds)
reg<-apply(gDat1[,id],1,function(x) sum(x>0))
if(!all(cDat[,1]==gDat1[,1])) stop("There is something wrong in the matching between cDat & gDat1")

if(length(pIds)!=length(id))  stop("there is something wrong with the pIds and the number of probesets use to compute the up and down regulation")

##==Updated: Oct 12, 2010	
##Association between predicted value and ocMOF<=3 vs ocMOF>3.
sink(file="log/information_paper.log",append=TRUE,split=TRUE)	
print("====================================================")
print(paste("The up/dowm of ocMOFi-iii vs iv-v for HLA-DR:",sep=""))
print(kruskal.test(c(reg),ifelse(cDat[,"cluster"]<=3,0,1)))
print("====================================================")
sink(file=NULL)

#ocMOF.col<-c(3:6,2)
#out<-boxplot(reg~cDat[,"cluster"],ylab="",xlab="",horizontal=FALSE,
#             boxwex=0.6,yaxt="n",col=ocMOF.col,names=NA,cex=(2/3))#,medcol="brown")
#axis(2, at = c(0,5,10,15), labels = c(0,5,10,15), tick = TRUE,las=1,cex.axis=1,lwd=0,lwd.ticks=1)
#axis(1, at = c(1:5), labels = ocMOF.label, tick = TRUE,las=1,cex.axis=1,lwd=0,lwd.ticks=1)
#
#mtext("ocMOF",side=1,line=2,cex=1.5)  
#mtext("Number of up-regulated",side=2,line=3,outer=FALSE,cex=1.5)
#mtext("probesets",side=2,line=2,outer=FALSE,cex=1.5)






