##====================================================================##
##==Purpose: To get all the clinical variables into one *.Rdata file==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *type1                                                        ==##
##==  A string indicating the gene expression of interest           ==##
##==  *Data associated with gene expressions/clinical variables     ==##                       
##==  "data/Rdata/microarray/",type1,".Rdata"                       ==##
##==  *Clinical variables                                           ==##
##==  -data/clinical_data/TRDB_TRAUMA-PATIENT_CLIN_DATA_RPT_20090126==##
##==  _195655.xls                                                   ==##
##==  -data/clinical_data/TRDB_TRAUMA-PATIENT_CALC_DATA_RPT_20090710==##
##==  _173925.xls                                                   ==##
##==  -data/clinical_data/TRDB_TRAUMA-MULT_ORGN_FAIL_RPT_20090708   ==##
##==  _153904.xls                                                   ==##
##==  -data/clinical_data/TRDB_TRAUMA-SAMPLE_DIFF_COUNTS_RPT_       ==##
##==  20090708_154716.xls                                           ==##
##==  -data/clinical_data/TRDB_TIME_VARY_TRAUMA_RPT_20090708_154807 ==##
##==  .xls                                                          ==##
##==  -data/clinical_data/TRDB_TRAUMA-RESUS_ENDPTS_INTERVNTN_RPT    ==##
##==  _20090708_154904.xls                                          ==##
##==  -data/Rdata/dat.cfr.Rdata                                     ==##
##==  dat.cfr, a data frame with columns corresponding to           ==## 
##==  patient_id, cluster (i.e. ocMOF),batch and sex.               ==##
##==  -data/Rdata/xDat(marshall_nogcs).Rdata                        ==##
##==  xDat, a data frame with rows corresponding to time, columns   ==##
##==  corresponding to patient_id, and the entries the mof score.   ==##
##==                                                                ==##
##==Output:                                                         ==##
##==   data/Rdata/clinDatInfo12_250hr.Rdata                         ==##
##==                                                                ==##
##== R libraries: gdata                                             ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##


library(gdata)

##==Filter the microarray data and order the patient ids according to the microarray data
##Load the slope matrix
load(file=paste("data/Rdata/microarray/",type1,".Rdata",sep=""))
##Remove the individuals due to quality reasons and death status
gDat<-gDat[,!(as.numeric(colnames(gDat))%in%c(8028773,25993337,33426107))]
##The slope matrix (WPEC) column order is imposed on the clinical matrix
ref<-colnames(gDat)


##==Read clinical data that are non-varying
clinDatNV<-read.xls("data/clinical_data/TRDB_TRAUMA-PATIENT_CLIN_DATA_RPT_20090126_195655.xls")
print("Reading TRDB_TRAUMA-PATIENT_CLIN_DATA_RPT_20090126_195655.xls")
print(dim(clinDatNV))
##Order the rows of data to gDat
mat<-clinDatNV[,"PATIENT_ID"]
id<-match(ref,mat)
clinDatNV<-clinDatNV[id,]
if(!all(mat<-clinDatNV[,"PATIENT_ID"]==ref)) stop("clinDatNV does not tally with gDat")


##==Append integrated MOF to clinDatNV
##Compute integrated MOF
##The time when MOF is computed/collected
time<-c(0,2:28)
##Load the MOF score
load(file=paste("data/Rdata/xDat(marshall_nogcs).Rdata",sep=""))
##Simple integration of the MOF scores (use trapesium rule):
imof2<-apply(xDat,2,function(y,x){
	
	id<-which(!is.na(y))
	if(length(id)<=2) stop("only have 2 or less MOF scores")
	diff.x<-c(diff(x[id]),1)
	return(sum(y[id]*diff.x)/(diff(range(x[id]))+1))
},x=time)
rm(xDat)
##Order the rows of data to gDat
mat<-names(imof2)
id<-match(ref,mat)
imof2<-imof2[id]
if(!all(names(imof2)==ref)) stop("imof2 does not tally with gDat")
clinDatNV<-data.frame(clinDatNV,imof2=imof2)


##==Append batch and ocMOF, and make them factors
load(file=paste("data/Rdata/dat.cfr.Rdata"))
##Order the rows of data to gDat
mat<-dat.cfr[,"patient_id"]
id<-match(ref,mat)
cDat<-dat.cfr[id,]
if(!all(cDat[,1]==ref)) stop("batch does not tally with gDat")
clinDatNV<-data.frame(clinDatNV,batch=factor(cDat[,"batch"]),ocMOF=as.factor(cDat[,"cluster"]))


##==Read clinical data that are non-varying and calculated
clinDatNVC<-read.xls("data/clinical_data/TRDB_TRAUMA-PATIENT_CALC_DATA_RPT_20090710_173925.xls")
print("Reading TRDB_TRAUMA-PATIENT_CALC_DATA_RPT_20090710_173925.xls")
print(dim(clinDatNVC))
##Order the rows of data to gDat
mat<-clinDatNVC[,"PATIENT_ID"]
id<-match(ref,mat)
clinDatNVC<-clinDatNVC[id,]
if(!all(mat<-clinDatNVC[,"PATIENT_ID"]==ref)) stop("clinDatNVC does not tally with gDat")

##==MOF info :Read clinical data to validate the groupings [Note that it has only 162 patients out of 168]
clinDatMOF<-read.xls("data/clinical_data/TRDB_TRAUMA-MULT_ORGN_FAIL_RPT_20090708_153904.xls")
##Get the data into wide form
wideDat<-reshape(clinDatMOF,v.names=colnames(clinDatMOF)[-c(1:3)],
                 idvar=colnames(clinDatMOF)[1],timevar=colnames(clinDatMOF)[2],
                 drop=colnames(clinDatMOF)[3],direction="wide")
print("Reading TRDB_TRAUMA-MULT_ORGN_FAIL_RPT_20090708_153904.xls")
print(dim(wideDat))
##Order the rows of data to gDat
mat<-wideDat[,"PATIENT_ID"]
tmp<-data.frame(matrix(NA,ncol=ncol(wideDat),nrow=nrow(clinDatNV)))
id<-match(ref,mat)
wideDat<-wideDat[id,]
if(!all(wideDat[!is.na(id),"PATIENT_ID"]==ref[!is.na(id)])) stop("clinDatMOF does not tally with gDat")
clinDatMOF<-wideDat
clinDatMOF[,1]<-ref

##==Blood counts :Read clinical data to validate the groupings
clinDatBC<-read.xls("data/clinical_data/TRDB_TRAUMA-SAMPLE_DIFF_COUNTS_RPT_20090708_154716.xls")
##Get the data into wide form
wideDat<-reshape(clinDatBC,v.names=colnames(clinDatBC)[-c(1:5)],
                 idvar=colnames(clinDatBC)[1],timevar=colnames(clinDatBC)[3],
                 drop=colnames(clinDatBC)[c(2,4,5)],direction="wide")
print("Reading TRDB_TRAUMA-SAMPLE_DIFF_COUNTS_RPT_20090708_154716.xls")
print(dim(wideDat))
##Order the rows of data to gDat
mat<-wideDat[,"PATIENT_ID"]
tmp<-data.frame(matrix(NA,ncol=ncol(wideDat),nrow=nrow(clinDatNV)))
id<-match(ref,mat)
wideDat<-wideDat[id,]
all(wideDat[!is.na(id),"PATIENT_ID"]==ref[!is.na(id)])
clinDatBC<-wideDat
clinDatBC[,1]<-ref


##==Time varying calculate field :Read clinical data to validate the groupings
clinDatTV<-read.xls("data/clinical_data/TRDB_TIME_VARY_TRAUMA_RPT_20090708_154807.xls")
##Get the data into wide form
wideDat<-reshape(clinDatTV,v.names=colnames(clinDatTV)[-c(1:2)],
                 idvar=colnames(clinDatTV)[1],timevar=colnames(clinDatTV)[2],
                 direction="wide")
print("Reading TRDB_TIME_VARY_TRAUMA_RPT_20090708_154807.xls")
print(dim(wideDat))
##Order the rows of data to gDat
mat<-wideDat[,"PATIENT_ID"]
tmp<-data.frame(matrix(NA,ncol=ncol(wideDat),nrow=nrow(clinDatNV)))
id<-match(ref,mat)
wideDat<-wideDat[id,]
if(!all(wideDat[!is.na(id),"PATIENT_ID"]==ref[!is.na(id)])) stop("clinDatTV does not tally with gDat")
clinDatTV<-wideDat
clinDatTV[,1]<-ref

##==Res :Read clinical data to validate the groupings
clinDatRES<-read.xls("data/clinical_data/TRDB_TRAUMA-RESUS_ENDPTS_INTERVNTN_RPT_20090708_154904.xls")
##Get the data into wide form
wideDat<-reshape(clinDatRES,v.names=colnames(clinDatRES)[-c(1:2)],
                 idvar=colnames(clinDatRES)[1],timevar=colnames(clinDatRES)[2],
                 direction="wide")
print("Reading TRDB_TRAUMA-RESUS_ENDPTS_INTERVNTN_RPT_20090708_154904.xls")
print(dim(wideDat))
##Order the rows of data to gDat
mat<-wideDat[,"PATIENT_ID"]
tmp<-data.frame(matrix(NA,ncol=ncol(wideDat),nrow=nrow(clinDatNV)))
id<-match(ref,mat)
wideDat<-wideDat[id,]
if(!all(wideDat[!is.na(id),"PATIENT_ID"]==ref[!is.na(id)])) stop("clinDatRES does not tally with gDat")
clinDatRES<-wideDat
clinDatRES[,1]<-ref

##Check if all are aligned
if(!all(clinDatNV[,"PATIENT_ID"]==ref))  stop("clinDatNV does not tally with gDat")
if(!all(clinDatNVC[,"PATIENT_ID"]==ref)) stop("clinDatNVC does not tally with gDat")
if(!all(clinDatTV[,"PATIENT_ID"]==ref))  stop("clinDatTV does not tally with gDat")
if(!all(clinDatMOF[,"PATIENT_ID"]==ref)) stop("clinDatMOF does not tally with gDat")
if(!all(clinDatBC[,"PATIENT_ID"]==ref))  stop("clinDatBC does not tally with gDat")
if(!all(clinDatRES[,"PATIENT_ID"]==ref)) stop("clinDatRES does not tally with gDat")


save(clinDatNV,clinDatNVC,clinDatTV,clinDatMOF,clinDatBC,clinDatRES,
     file="data/Rdata/clinDatInfo12_250hr.Rdata")

