##====================================================================##
##==Purpose: Regresses the clinical variable onto the principal     ==##
##==         components of WPEC/mean expression/estimated expression==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *gDat, a matrix with columns corresponding to patients and    ==##
##==  rows the probests. The entries are the related to gene        ==##
##==  expressions.                                                  ==##                                                          
##==  *svd.obj, the svd object of gDat after applying svd           ==##
##==  *data/Rdata/clinDatInfo12_250hr(clean).Rdata                  ==##
##==  -clinDat, a dataframe with columns corresponding to clinical  ==##
##==   variables and rows the patients                              ==##
##==  -pcall, a vector indicating the percentage of missing in the  ==##
##==   each column.                                                 ==##             
##==  -datType, a vector indicating which data source the clinical  ==##
##==   variable came from.                                          ==##
##==                                                                ==##
##==R library: nnet, pscl, qvalue                                   ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##

library(nnet)
library(pscl)
library(qvalue)

##==Order the patient_id according to microarray
ref<-colnames(gDat)
##Order the clinical variables
load(file="data/Rdata/clinDatInfo12_250hr(clean).Rdata")
mat<-clinDat[,1]
id<-match(ref,mat)
clinDat<-clinDat[id,]
if(!all(clinDat[,1]==ref)) stop("Stop the datasets do not match up")

##==Assess the structure of the micrarray data by correlating the principal components with clinical variables
##Get the right singular values (i.e. principal components)
V<-svd.obj$v[,1:no.comp]

##Using linear regression to determine association between slope expression and continuous clinical variables
id.nf<-which(sapply(clinDat,class)!="factor")[-1]
#print(summary(clinDat[,id.nf]))

##Identify those variables with less than or equal to 5 variables
id.nf5<-id.nf[apply(clinDat[,id.nf],2,function(x) length(unique(x[!is.na(x)]))<=5)]
##Remove those with 5 or less unique values
id.nf<-id.nf[!(id.nf%in%id.nf5)]
#clinDat[,id.nf5[7:8]]

print(type1)
print("Continuous Clinical Variables")
p.sp<-NULL
count<-1
p.sp.7pval<-p.sp.7coef<-NULL
r2.sp<-NULL

for(i in colnames(clinDat)[id.nf]){
  
  ##Generate the data for regressing clinical variable on principal components	
  tmp<-data.frame(y=unlist(clinDat[,i]),V)
  tmp<-tmp[!apply(is.na(tmp),1,any),]
  tmp[,"y"]<-qnorm((rank(tmp[,"y"])-1/3)/(nrow(tmp)+1/4))	
  ##Generate the statistics	
  model1<-lm(y~.,data=tmp)
  f.stat<-summary(model1)$fstatistic
  p.tmp<-1-pf(f.stat[1],df1=f.stat[2],df2=f.stat[3])
  p.sp<-c(p.sp,p.tmp)
  r2.sp<-c(r2.sp,summary(model1)$r.squared)
  names(p.sp)[count]<-i  
  ##Get pvals and coef of each compoent
  p.sp.7pval<-cbind(p.sp.7pval,summary(model1)$coefficients[2:(no.comp+1),4]) 
  p.sp.7coef<-cbind(p.sp.7coef,summary(model1)$coefficients[2:(no.comp+1),1]) 

  cat("\r","Clinical variable:",i)
	
  count<-count+1
}
cat("\n")
names(r2.sp)<-colnames(p.sp.7pval)<-colnames(p.sp.7coef)<-names(p.sp)

##Using multinomial regression model to determine association between slope expression and categorical-like clinical variables
id.factor<-which(sapply(clinDat,class)=="factor")
#print(summary(clinDat[,id.factor]))
##Include those continuous variables with 5 or less unique values
id.factor<-sort(c(id.factor,id.nf5))

print("Categorical Clinical Variables")
pKW<-NULL
resKW<-NULL
count<-1
converge0<-converge1<-NULL
pKW.7pval<-pKW.7coef<-NULL
r2KW<-NULL

for(i in colnames(clinDat)[id.factor]){

  ##Generate the data for regressing clinical variable on principal components	
  tmp<-data.frame(y=unlist(clinDat[,i]),V)
  tmp<-tmp[!apply(is.na(tmp),1,any),]
  ##Generate the statistics	
  model1<-multinom(y~.,data=tmp,maxit=10000000)
  r2KW<-cbind(r2KW,pR2(model1))
  model0<-multinom(y~1,data=tmp,maxit=10000000)

  converge1<-c(converge1,model1$convergence)
  converge0<-c(converge0,model0$convergence)
  df<-length(summary(model1)$coefficients)-length(summary(model0)$coefficients)
  p.tmp<-1-pchisq(model0$deviance-model1$deviance,df=df)
  pKW<-c(pKW,p.tmp)
  names(pKW)[count]<-i
	
  cat("\r","Clinical variable:",i) 
	
  count<-count+1
}
cat("\n")

print("================================================")
print("Make sure all are 0 for convergence of MNL model")
print(table(converge0))
print(table(converge1))
if(!all(converge0==0)) stop("Convergence was not reached")
if(!all(converge1==0)) stop("Convergence was not reached")
print("================================================")

##==Get the qvalue of the pvalues 
p.valc<-c(p.sp,pKW)
qvalc<-qvalue(p.valc)
#pdf(paste("plots/forpaper/p_value_clinical",type1,".pdf",sep=""))
#par(mfrow=c(1,1))
#hist(p.valc,ylab="P-value")
#dev.off()

##==Consolidate the pvalue, qvalue and missing info:
no.na<-apply(is.na(clinDat),2,sum)
no.na<-no.na[-1]
ref<-names(p.valc)
mat<-names(no.na)
idt<-match(ref,mat)
no.na<-no.na[idt]
if(!all(names(no.na)==ref)) stop("Something wrong in matching")

info<-data.frame(variable=names(p.valc),p.val=p.valc,q.val=qvalc$qvalues,per.not.na=1-no.na/nrow(clinDat),R2=c(r2.sp,r2KW["McFadden",]),factor=names(p.valc)%in%colnames(clinDat)[id.factor])
info<-info[order(info[,2]),]
rownames(info)<-NULL
