##====================================================================##
##==Purpose: Master script to generate all results and figures      ==##
##==         reported in the paper                      	    ==##
##==                                                                ==##
##==The workflow of this script:                                    ==##
##==  (1) Get the ordered categorical MOF (ocMOF)                   ==##
##==  (2) Normalize the microarrays separately according to batch   ==##
##==      with dchip and compute WPEC                               ==##
##==  (3) Perform adjusted Spearman analysis with WPEC and ocMOF    ==##
##==  (4) Perform reproducibility analysis using 20                 ==##
##==      cross-validations                                         ==##
##==  (5) Investigate the modules and gene sets discussed in the    ==##
##==      paper                                                     ==##
##==  (6) Assessing the microarrays in the GLUE data                ==##
##==                                                                ==##
##==The output of interest:                                         ==##
##==  (1) log, a subfolder containing information either mentioned  ==##
##==      or presented as a table in the paper or supplementary     ==##
##==      material (information_paper.log) and the output of the R  ==##
##==      console (main.log)                                        ==##
##==  (2) plots/forpaper, a subfolder containing the figures used   ==##
##==      in the paper and supplementary material                   ==##
##==  (3) tables, a subfolder containing large tables presented in  ==##
##==      the paper or supplementary material                       ==##
##==                                                                ==##
##==Updated:                                                        ==##
##==  *Oct 12, 2010:                                                ==##
##==  1. Patient characteristics of the various ocMOF sub-groups    ==##
##==     are expanded to include age and etc. Now this information  ==##
##==     is available at code/6_assessing_data/ClinInfoTable.R      ==##
##==  2. Included the association between slope and ocMOF, and the  ==##
##==     association between predicted value and ocMOF<=3 vs        ==##
##==     ocMOF>3 in slope_trajectory_count.R in                     ==##
##==     code/5_genesets_pred126pat                                 ==##
##==  *Jun 10, 2011:                                                ==##
##==  1. Update the Supplemenatry Table with average WPEC for ocMOF ==##
##==     subgroups.                                                 ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##


##==If the main.R is sourced by the user, we need to set working directory and specify the options/settings
if(!any(ls()%in%"runAnalysis")){
  ##==If main.R is sourced directly instead of using runAnalysis.Rdata
  ##==This will automatically start the analysis in R (if source("code/main.R") is used)

  settings<-function(){
    
    enum <- "\n Enter number: "
    evar <- "\n Enter working directory: "
    cat("
		To run the analysis we have performed, we
		will ask you a series of questions to set up 
		the analysis. See Information.rtf in the 
		documentation subfolder for more information.
		Press Escape at any time to abort. Since you
		are running the code by sourcing the main.R
		file, we are assuming that you are familar 
		with R.
		\n")
	
    ##The R scripts are running via runAnalysis.R
    runAnalysis <- TRUE
    check.values <- function(x,values,msg){
      if((x %in% values)){
        invisible(x)
      }
      else{
        while(!(x %in% values)){
          cat("\n \t Please select an appropriate value, or press Escape to abort\n")
          x <- readline(msg)
        }
      }
    }
    
	
    # Early abort check
    msg1 <- "\n Please choose one of the following options:\n1 -- To continue with the analysis\n2 -- To exit\n" 
    cat(msg1)
    abort <- readline(enum)
    abort <- check.values(abort,c("1","2"),enum)
    if(abort=="2"){return(invisible(1))}
    
    cat("\n------\n")
    
    # Specify the working directory
    msg <- "\n What is working directory? Do not include the quotation marks (i.e. ~/Archive). This information will be used by the function setwd().\n"
    cat(msg)
    wkdir <- readline(evar)
    chk.wkdir<-try(setwd(wkdir),silent=TRUE)
    while(inherits(chk.wkdir, "try-error")){
      cat("
			The working directory which you just entered 
			does not exist. Please check and enter again.
			(If this problem persists, consider running the
			analysis by double clicking on runAnalysis.Rdata.
			See Information.rtf in the documentation subfolder 
			for more information.")
      cat("
			------\n\n")		
      msg <- "\n Enter the working directory again. Press Escape to abort.\n"
      cat(msg)
      wkdir <- readline(evar)
      chk.wkdir<-try(setwd(wkdir),silent=TRUE)
    }
    
	
    # Normalize the microarray locally if windows
	if(.Platform$OS.type=="windows"){  
		  msg <- "\n You are on a windows machine, do you want to perform dChip normalization on your PC? Please note that this will take 2 or more days to finish the normalization. \n 1 -- Yes \n 2 -- No, I do not want to perform normalization on my PC\n"
		  cat(msg)
		  bypassnorm <- readline(enum)
		  bypassnorm <- check.values(bypassnorm,c("1","2"),enum)
		  
		  bypassnorm <- !(bypassnorm==1)
		  
		  cat("\n------\n")
	}else{
		  
		  bypassnorm <- TRUE
		
	}
	  
    # Plot a png file
    msg <- "\n If you are on a computer machine that allows you to save figures as *.png files, do you want to plot the gene sets figures in png?\n If you are unsure, please enter the number 2.\n 1 -- Yes \n 2 -- No, I do not want to plot these figures in png, or I am unsure if my computer allows me to save figures as *.png files\n"
    cat(msg)
    pngPlot <- readline(enum)
    pngPlot <- check.values(pngPlot,c("1","2"),enum)
	
    pngPlot <- (pngPlot==1)
    
    cat("\n------\n")

    # Create the directory for storing the data/analysis/log files/figures/tables
    ##For data
	dir.create("data/csv")
	dir.create("data/Rdata")
	dir.create("data/Rdata/microarray")
    ##For log files
	dir.create("log")
    ##For plots
	dir.create("plots")
	dir.create("plots/forpaper")
    ##For large tables
	dir.create("tables")
	  
    # Save the R object settings to settings.Rdata
    save(bypassnorm,pngPlot,file="data/Rdata/settings.Rdata")

  }
  settings()
  load(file="data/Rdata/settings.Rdata")
  ##==Specifying the working directory
#  wkdir<-"~/Glue/Archive"
#  ##==Specifying whether to normalize with the local machine
#  bypassnorm<-TRUE
#  ##==Specifying whether to plot some of the graphs in png
#  pngPlot<-FALSE
	
}else{
	
  ##==Create the directory for storing the data/analysis/log files/figures/tables
  ##For data
  dir.create("data/csv")
  dir.create("data/Rdata")
  dir.create("data/Rdata/microarray")
  ##For log files
  dir.create("log")
  ##For plots
  dir.create("plots")
  dir.create("plots/forpaper")
  ##For large tables
  dir.create("tables")

  ##==Save the R object settings to settings.Rdata
  save(bypassnorm,pngPlot,file="data/Rdata/settings.Rdata")
	
}

##==The R packages used in the paper
pkgs<-c("gdata", "gplots", "impute", "nnet", "pscl", "qvalue")
downpkgs<-c(library("gdata",logical=TRUE),
            library("gplots",logical=TRUE),
            library("impute",logical=TRUE),
            library("nnet",logical=TRUE),
            library("pscl",logical=TRUE),
            library("qvalue",logical=TRUE))
##To download R packages if they are not in the local computer
if(any(!downpkgs)){
  install.packages(pkgs[!downpkgs],repos="http://cran.r-project.org")
}
rm(pkgs)




##Diverts R output to a file
sink(file=ifelse(bypassnorm,"log/main_bypass.log","log/main.log"),append=FALSE,split=TRUE)

##....................................................##
##....................................................##
## (1) Get the ordered categorical MOF (ocMOF)        ##
##....................................................##
##....................................................##

##==Get the ocMOF and the plots in the Supplementary Material
load(file="data/Rdata/settings.Rdata")
source("code/1_ocMOF/Getocmof.R",local=TRUE)
rm(list=ls())


##....................................................##
##....................................................##
## (2) Normalize the microarrays separately according ##
##     to batch with dchip and compute WPEC           ##
##....................................................##
##....................................................##
load(file="data/Rdata/settings.Rdata")
if(!bypassnorm){
  print("Normalizing main dataset (0-250 hours). Time required is approximately 2 hours...")
  ##======================================##
  ##==Call dChip from within R to         ## 
  ##==normalize microarrays (CEL files) ==##
  ##==(Approximately 2 hours)           ==##
  ##======================================##

  ##Specify the working directory with backslash (for Microsoft Windows)
  wkdir.backslash<-paste(strsplit(getwd(),split="/")[[1]],collapse="\\")
  txt.info<-"microarrayID250.168.batch"
  source("code/2_normalization/DChipnormalization.R",local=TRUE)


  print("Normalizing dataset (0-800 hours). Time required is approximately 2 hours...")
  ##======================================##
  ##==Call dChip from within R to         ## 
  ##==normalize microarrays (CEL files) ==##
  ##==(Approximately 2 hours)           ==##
  ##======================================##
  
  ##Specify the working directory with backslash (for Microsoft Windows)
  wkdir.backslash<-paste(strsplit(getwd(),split="/")[[1]],collapse="\\")
  txt.info<-"microarrayIDALL.168.batch"
  source("code/2_normalization/DChipnormalization.R",local=TRUE)
  
  
  print("Normalizing 20 crossvalidation datasets. Time required is approximately 24 hours...")
  ##======================================##
  ##==Call dChip from within R to         ## 
  ##==normalize microarrays (CEL files) ==##
  ##==for 20 crossvalidation datasets   ==##
  ##==(Approximately 24 hours)          ==##
  ##======================================##
  ##==Divide 126 patients into 20 crossvalidation sets
  min_hours = 12
  max_hours = 250
  source("code/2_normalization/Splitdata.R",local=TRUE)
  rm(list=ls())
  
  ##==Normalize subset 1 in 20 crossvalidation sets
  ##Specify the working directory with backslash (for Microsoft Windows)
  wkdir.backslash<-paste(strsplit(getwd(),split="/")[[1]],collapse="\\")
  ##For training data (i.e. subset 1)
  for(split.id in 1:20){
    txt.info<-paste("split",split.id,"_subset1_phase",sep="")
    source("code/2_normalization/DChipnormalization.R",local=TRUE)
  }
  rm(list=ls())
  
  ##==Normalize subset 2 in 20 crossvalidation sets
  ##Specify the working directory with backslash
  wkdir.backslash<-paste(strsplit(getwd(),split="/")[[1]],collapse="\\")
  ##For test data (i.e. subset 2)
  for(split.id in 1:20){
    txt.info<-paste("split",split.id,"_subset2_phase",sep="")
    source("code/2_normalization/DChipnormalization.R",local=TRUE)
  }
  rm(list=ls())
  print("Normalization done...")
  
}

##======================================##
##==Consolidate microarray data &     ==##
##==clinical information from four    ==## 
##==separate batches into one big     ==##
##==matrix                            ==##
##======================================##
##To determine whether the normalization step is performed
min_hours<-12
max_hours<-250
txt.info<-"microarrayID250.168.batch"
load(file="data/Rdata/settings.Rdata")
source("code/2_normalization/Consolidatebatches.R",local=TRUE)
no.probesets<-nrow(Xraw)
rm(list=ls()[ls()!="no.probesets"])

print("Data from 4 separate batches consolidated...")

##==The number of microarrays collected and used in this analysis
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
##Microarrays collected from hour 0 to 800 and no filtering based on RNA quality
info.all<-read.table(file="data/CEL/info.all.csv",sep=",")	
print("==========================================================")
print(paste(length(unique(info.all[,"patient.id"]))," patients, 0-800 hrs without filtering on rna quality",sep=""))
print(paste("Total number of microarrays collected:",nrow(info.all)))
print("The range of microarrays collected for each patient:")
print(range(table(info.all[,"patient.id"])))
rm(info.all)

##Microarrays collected from hour 0 to 250 and with RNA quality >1
info.250<-read.table(file="data/CEL/info.250.csv",sep=",")
print(paste(length(unique(info.250[,"patient.id"]))," patients, 0-250 hrs with with rna_quality>1",sep=""))
print(paste("Total number of microarrays normalized:",nrow(info.250)))

##The number of microarrays used for computing WPEC
valid<-which(info.250[,"hours.si"]>=12 & info.250[,"hours.si"]<=250)
tmp<-as.data.frame(table(info.250[valid,"patient.id"]))
tmp<-tmp[tmp[,2]>2,]

print(paste(nrow(tmp)," patients' WPEC information was obtained for 12-250 hrs with rna_quality>1 and >2 arrays",sep=""))
print(paste("Total number of normalized microarrays used:",sum(tmp[,2])))

##The number of microarrays used for final analysis
tmp1<-tmp[!(tmp[,1]%in%c(8028773,25993337,33426107)),]
print(paste(nrow(tmp1)," patients were used in the adjusted Spearman analysis",sep=""))
print(paste("Total number of normalized microarrays used:",sum(tmp1[,2])))
print(paste("Total number of probesets:",no.probesets))
print("==========================================================")
rm(list=ls())
sink(file=NULL)

##======================================##
##==Generate the WPEC matrix from     ==##
##==12-250 hours                      ==##
##======================================##
txt.info<-"microarrayID250.168.batch"
constant<-10
min_hours<-12
max_hours<-250
#Summary method (0=LR Slope, 1=LR output intercept, 2=Mean value)
method<-0
source("code/2_normalization/ComputeWPEC.R",local=TRUE)
rm(list=ls())

print("WPEC calculated and stored in .Rdata file...") 


##....................................................##
##....................................................##
## (3) Perform adjusted Spearman analysis with WPEC   ##
##     and ocMOF                                      ##
##....................................................##
##....................................................##

print("Associating WPEC with ocMOF. Time required is approximately 3 hours...")
##======================================##
##==Associate WPEC with ocMOF and     ==##
##==generate the results in Fig. 2 of ==##
##==the paper and a compact table     ==##
##==containing top 500 probesets      ==##
##======================================##
##==Perform the adjusted Spearman analysis 
txt.info<-"microarrayID250.168.batch"
min_hours<-12;
max_hours<-250;
type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")

source("code/3_analysis/Adjanalysis.R",local=TRUE)
#rm(list=ls())

##==Get the ranking of all probesets into a table, Supplementary_Table4
txt.info<-"microarrayID250.168.batch"
min_hours<-12;
max_hours<-250;
type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")
top<-  read.table(file=paste("data/csv/res.",type1,"adj.csv",sep=""),sep=",")
top<-top[,-4]
colnames(top)<-c("Probeset ID","P-value","Q-value")
write.table(top,file=paste("tables/Supplementary_Table3_55kprobesets.csv",sep=""),sep=",",row.names=FALSE)

#added on 10 June 2011 to update the Supplemenatry Table with average WPEC for ocMOF subgroups
#requires gDat and cDat from Adjanalysis.R
########################################
top = read.csv(file=paste("tables/Supplementary_Table3_55kprobesets.csv",sep=""),sep=",")
wpecavg = NULL
for(ii in 1:5){
tmp = apply(gDat[,cDat[,"cluster"]==ii],1,mean)
wpecavg = cbind(wpecavg,tmp)
}
colnames(wpecavg) <- c("avgwpec.ocMOF1","avgwpec.ocMOF2","avgwpec.ocMOF3","avgwpec.ocMOF4","avgwpec.ocMOF5")
idx = match(top[,1],rownames(wpecavg))
wpecavg.reordered = wpecavg[idx,]
top = cbind(top,wpecavg.reordered)
write.table(top,file=paste("tables/Supplementary_Table3_55kprobesets.csv",sep=""),sep=",",row.names=FALSE)
###########################################

rm(list=ls())


##....................................................##
##....................................................##
## (4) Perform reproducibility analysis using         ##
##     20 cross-validations                           ##
##....................................................##
##....................................................##

print("Assessing reproducibility. Time required is approximately 3 hours...")

##==Reproducibility analysis via cross-validation
##To determine whether the normalization step is by-passed
source("code/4_reprod_20cv/Reproducibility.R")
rm(list=ls())


##....................................................##
##....................................................##
## (5) Investigate the modules and gene sets          ##
##     discussed in the paper                         ##
##....................................................##
##....................................................##


##==Get the burn microarray information for weighting the probesets
source("code/5_genesets/Array_days.R")
rm(list=ls())

##==Module approach from 3663 probesets
source("code/5_genesets/ModuleTop3663.R",local=TRUE)
rm(list=ls())

##==Get the probesets of the gene sets (canonical pathway/DAVID) and their expression trajectories of each probeset
constant<-10
txt.info<-"microarrayID250.168.batch"
min_hours<-12;
max_hours<-250;
type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")

source("code/5_genesets/Probesets_genesets.R",local=TRUE)
rm(list=ls())

##==Plot the figures for the main paper
source("code/5_genesets/Dominant_trajectory.R")
rm(list=ls())

##==Plot the figures for the supplementary paper
source("code/5_genesets/Boxplot_WPEC.R")
rm(list=ls())

##==The endotoxin data analysis in the supplementary paper
source("code/5_genesets/Endotoxin_analysis.R")
rm(list=ls())

##....................................................##
##....................................................##
## (6) Assessing the microarrays in the GLUE data     ##
##....................................................##
##....................................................##

##==Updated: Oct 12, 2010
##==Patient characteristics of the various ocMOF sub-groups. Information for supplementary appendix
source("code/6_assessing_data/ClinInfoTable.R",local=TRUE)
rm(list=ls())

##==Heatmap of 797 microarrays
##To determine whether the normalization step is by-passed
load(file="data/Rdata/settings.Rdata")
source("code/6_assessing_data/HeatmapAll.R",local=TRUE)

##==Illustrating microarray collection and data quality issues  
source("code/6_assessing_data/DataQualArrayTime.R")
rm(list=ls())

##==Reason for excluding the first 12 hours gene expression
source("code/6_assessing_data/First12hour.R")
rm(list=ls())

##==Reading in the clinical variables from GLUE for assessing the microarray data
txt.info<-"microarrayID250.168.batch"
min_hours<-12;
max_hours<-250;
type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")
source("code/6_assessing_data/ReadClinVar.R",local=TRUE)
rm(list=ls())

source("code/6_assessing_data/CleanClinVar.R",local=TRUE)
rm(list=ls())

##==Regressing the clinical variables on the principal components of expression matrices
source("code/6_assessing_data/PCAarrayClinVarWPEC.R")
rm(list=ls())

source("code/6_assessing_data/PCAarrayClinVarMean.R")
rm(list=ls())

##==Assessing the up/down counting of HLA-DR probesets within the MHC-II gene sets
source("code/6_assessing_data/HLA-DR.R")

sink(file=NULL)
